import type Viewer from "@/lib/three/Viewer";
import * as THREE from "three";
import { AmbientLight, DirectionalLight, Mesh } from "three";

// 添加立方体
export default class BoxGeometry {
  protected viewer: Viewer;
  public cube!: Mesh;
  public directionalLight!: DirectionalLight;

  constructor(viewer: Viewer) {
    this.viewer = viewer;
    this.addBoxGeometry();
    this.initLight();
    this.initPlane();
  }

  private initLight() {
    const material = new THREE.MeshStandardMaterial({ color: 0xff0000 });
    // 添加环境光
    const ambientLight = new AmbientLight(0xffffff, 0.4);
    this.viewer.scene.add(ambientLight);

    this.directionalLight = new THREE.DirectionalLight(0xffffff, 1);
    this.directionalLight.position.set(10, 0, 10);
    this.viewer.scene.add(this.directionalLight);
  }

  private initPlane() {
    this.viewer.renderer.shadowMap.enabled = true; // 1. 渲染器能够渲染阴影效果
    this.directionalLight.castShadow = true; // 2. 该方向会投射阴影效果
    this.cube.castShadow = true;

    const planeGeometry = new THREE.PlaneGeometry(20, 20);
    const planeMaterial = new THREE.MeshStandardMaterial({ color: 0xffffff });
    const planeMesh = new THREE.Mesh(planeGeometry, planeMaterial);
    planeMesh.rotation.x = -0.5 * Math.PI;
    planeMesh.position.set(0, -3, 0);
    planeMesh.receiveShadow = true;
    this.viewer.scene.add(planeMesh);

    const directionalLightHelper = new THREE.DirectionalLightHelper(this.directionalLight);
    this.viewer.scene.add(directionalLightHelper);
  }

  public addBoxGeometry() {
    // 创建长宽高都为4的立方几何体；
    // 立方体的表明颜色为红色的材质;
    // 将集合体和材质组合为Mesh对象；
    // 将Mesh对象添加到场景中。
    const geometry = new THREE.BoxGeometry(4, 4, 4);
    const material = new THREE.MeshBasicMaterial({ color: 0xff0000 });
    this.cube = new THREE.Mesh(geometry, material);

    this.viewer.scene.add(this.cube);
  }
}
