import React, { FC, useEffect, useRef, useState } from "react";
import { observer } from "mobx-react";
import { useMobStore } from "@/hooks";
import * as THREE from "three";
import Viewer from "./modules/Viewer";
import ModelLoader from "./modules/ModelLoder";
import BoxHelperWrap from "./modules/BoxHelperWrap";
import { checkNameIncludes } from "@/utils/three";
import Floors from "./modules/Floors";
import { Object3DExtends } from "@/types";
import styles from "./index.less";

interface OwnProps {}

type Props = OwnProps;

const PAGE_ID = "DEMO_CONTAINER";
const ThreeJSIndex: FC<Props> = observer((props) => {
  const store = useMobStore("threeStore");
  const [rackList, setRackList] = useState<Object3DExtends[]>([]); //架子
  const [chairList, setChairList] = useState<Object3DExtends[]>([]); //椅子
  const viewerRef = useRef<Viewer>();
  let modelLoader: ModelLoader;
  let boxHelperWrap: BoxHelperWrap;
  useEffect(() => {
    init();
    initModel();
    return () => viewerRef.current?.destroy();
  }, []);
  /** 需要监听rackInfoList更新监听点击事件的函数 */
  useEffect(() => {
    if (!viewerRef.current) return;
    if (_.isEmpty(store?.raycasterObjects)) return;
    const viewer = viewerRef.current;
    onMouseClick(store?.raycasterObjects);
  }, [store?.raycasterObjects, viewerRef, viewerRef.current?.scene]);
  // 加载
  const init = () => {
    viewerRef.current = new Viewer(PAGE_ID);
    const viewer = viewerRef.current;
    viewer.addAxis();
    viewer.addStats();
    viewer.initRaycaster();

    modelLoader = new ModelLoader(viewer);
    const floors = new Floors(viewer);
    floors.addGird(8, 25, 0x004444, 0x004444);
    // boxHelperWrap = new BoxHelperWrap(viewer);
  };
  // 加载模型
  const initModel = () => {
    modelLoader.loadModelToScene("/models/datacenter.glb", (baseModel) => {
      // 设置基础模型的缩放比例
      baseModel.setScalc(0.15);
      // 暂时注释掉旋转代码
      // baseModel.object.rotation.y = Math.PI / 2;
      // 获取实际的模型对象
      const model = baseModel.gltf.scene;
      model.position.set(0, 0, 0.3);
      // 为模型设置名称
      model.name = "机房";
      model.uuid = "机房";
      // 启用基础模型的投射阴影功能
      baseModel.openCastShadow();
      let rackList: Object3DExtends[] = []; // 机架列表
      let allList: Object3DExtends[] = []; // 所有的物体
      let chairList: Object3DExtends[] = []; // 椅子列表
      model.traverse((item) => {
        if (checkIsRack(item)) {
          rackList.push(item);
        }
        if (checkIsChair(item)) {
          chairList.push(item);
        }
        allList.push(item);
        if (item instanceof THREE.Mesh) {
          // 保存原始颜色数据，以及警告颜色
          if (item.isMesh) {
            item.material.warningColor = { r: 1, g: 0, b: 0, isColor: true };
            // 保存旧的材质
            (item as Object3DExtends).oldMaterial = item.material;
          }
        }
      });
      // dispatchDeviceListData({ type: "INIT", initData: rackList });
      setRackList(rackList);
      setChairList(chairList);
      const viewer = viewerRef.current;
      // 将 rackList 中的机架设置为 viewer 的射线检测对象
      viewer?.setRaycasterObjects([...allList]);
    });
  };
  const checkIsRack = (obj: THREE.Object3D): boolean => {
    return checkNameIncludes(obj, "rack");
  };
  const checkIsChair = (obj: THREE.Object3D): boolean => {
    return checkNameIncludes(obj, "chair");
  };
  const onClickChair = (selectedObject: THREE.Object3D) => {
    if (!checkNameIncludes(selectedObject, "chair")) return;
    const viewer = viewerRef.current;

    viewer?.addCameraTween(new THREE.Vector3(0.05, 0.66, -2.54), 1000, () => {
      console.log("动画完成");
      console.log(viewer?.scene?.children);
      const sceneModels = viewer?.scene?.children;
      // const model = sceneModels?.find((model) => {
      //   return model.name === "机房";
      // })!;
      // const floorModel = sceneModels?.find((model) => {
      //   return model.name === "机房";
      // })
      viewer?.setRaycasterObjects([]); //
      sceneModels?.forEach((model) => {
        viewer?.scene.remove(model);
      });

      // createRoom("room", new THREE.Vector3(0, 0, 0));
    });
  };
  const onMouseClick = (intersects: THREE.Intersection[]) => {
    if (!intersects.length) return;
    const selectedObject = intersects?.[0].object || {};
    // onClickRack(selectedObject);
    onClickChair(selectedObject);
  };

  return (
    <div className={styles.threejs_container}>
      <div id={PAGE_ID} />
    </div>
  );
});

export default ThreeJSIndex;
