import type { Material } from "three";
import * as THREE from "three";
import type Viewer from "../Viewer";
import type { GLTF } from "three/examples/jsm/loaders/GLTFLoader.js";
import { Object3DExtends } from "@/types";

export default class BaseModel {
  protected viewer: Viewer;
  public gltf: GLTF; // 模型
  public object: THREE.Group;
  /**模型原始材质 */
  public originMaterials: Material[] = [];
  public isSaveMaterial = false;
  public animaIndex = -1;
  public mixer!: THREE.AnimationMixer;
  public clock: THREE.Clock;

  constructor(gltf: GLTF, viewer: Viewer) {
    this.gltf = gltf;
    this.viewer = viewer;
    this.object = gltf.scene || gltf;

    this.clock = new THREE.Clock(); // 时钟
  }

  /**
   * 设置模型比例
   * @param x 可以只填写一个参数
   * @param y 纵轴缩放
   * @param z 横轴缩放
   */
  public setScalc(x: number, y?: number, z?: number) {
    this.object.scale.set(x, y || x, z || x);
  }

  /**
   * 开启模型阴影 数组中移除阴影
   */
  public openCastShadow(names = []) {
    this.gltf.scene.traverse((model: Object3DExtends) => {
      if (model.isMesh && !names.includes(model.name as never)) {
        //它会在渲染对象之前检查每一帧对象是否位于相机的视锥体中。 如果设置为 false，则即使对象不在相机的视锥体中，也会在每一帧进行渲染。
        model.frustumCulled = false;
        model.castShadow = true; //对象是否渲染成阴影贴图。
      }
    });
  }
}
